# Datenbanken erstellen

*Dateiname: instaHub-passwoerter.md*

Datenbanken stellen eine Mini-Welt der Realität dar (Reduktion). Für die Konzeption einer Datenbank ist daher vorab zu überlegen, welche der Informationen von Relevanz für den Sachverhalt sind.

Mit Hilfe der Datenbank sollen jene relevante Informationen für uns in strukturierter Form zugänglich gemacht werden.

Um das "Chaos" im Griff zu halten, muss man schrittweise vorgehen. Einige Dinge sind zudem in einem ersten Schritt nicht notwendig:

- Attribute spielen in der Regel noch keine so große Rolle und müssen noch nicht vollständig sein. 
- Die Datentypen der Attribute sind für die Modellierung zunächst nicht so wichtig.
- Das Relationale Datenmodell erfordert Ergänzungen (z.B. Schlüssel), die für das Modell der Miniwelt selbst nicht notwendig sind.

Wichtig ist zunächst, den Ausschnitt der realen Welt festzulegen, der auch im Datenmodell erfasst werden soll. Man spricht auch von der sog. modellierten "Miniwelt". Der Entwurf einer relationalen Datenbank erfolgt dann in zwei Schritten:

![Datenbank Konzeption](bilder/datenbank-konzeption.png)

Die Miniwelt wird abgegrenzt und für diese dann ein vereinfachtes konzeptionelles Modell entworfen. Wir verwenden zur Darstellung das sog. **ER-Modell (Entity-Relationship-Modell)**. Auch andere Darstellungsformen (z.B. RM, UML) sind möglich.

### Worum geht es eigentlich?


## ER-Modell

i>Das **Entity-Relationship-Modell* (kurz ERM) dient zur systematischen Modellierungen einer Datenbank. Gleichzeitig ist es als *universelle Sprache* als Grundlage für viele Datenbanksysteme nutzbar. Grundsätzlich werden Objekte (Entitys) und ihre Beziehungen (Relations) dargestellt.

### Begriffe zum ERM

Begriff | Erklärung | Beispiel | Kennzeichnung im ERM
--------|-----------|----------|---------------------
Entity (Datensatz) |Unterscheidbare Dinge aus der realen Welt ; unterscheiden sich voneinander in mindestens einem Eigenschaftswert | Abteilung<br />Forschung <br />Mitarbeiter <br />Schmidt <br />Projekt 109 | 
Klassen / Entitätsmenge (Tabelle) | Sammlung gleichartiger Entities (gleiche Eigenschaften, unterschiedliche Merkmale) | Mitarbeiter | Rechteck
Attribut (Spalte in Tabelle) | Merkmal & Eigenschaft gleichartiger Objekte | Name, Vorname | Ellipse
Primärschlüssel | Ein oder mehrere Attribute, welche Objekt eindeutig definieren | Name | unterstrichenes Attribut
Beziehung | Drücken Wechselwirkung aus; können ebenfalls Attribute besitzen | Schüler **besucht** Klasse | Raute
Assoziationstypen/Kardinalitäten | Typ der Beziehung zwwischen zwei Klassen gleichartiger Objekte | 1:1<br />1:n<br />m:n | als Notation an Relation

### Hilfestellung

*Regeln nach Peter Chen für die ERM-Erstellung*

1. Entitäten und Beziehungen identifizieren
2. Attribute ermitteln
3. Primärschlüssel für Entitäten bestimmen | Fremdschlüssel bestimmen
4. Festlegen der Beziehungskardinalitäten
5. Entity-Relationship-Diagramme (ERD) für Entitäts- und Beziehungsmengen zeichnen


### Beispiel

![ERM Beispiel](bilder/erm-beispiel.png)

t>Beschreibe die vorliegende Datenbank mit eigenen Worten.

---

## Kardinalitäten

i>**1:1-Beziehung**<br />Einem Element der Menge A wird genau ein Element der Menge B zugeordnet.

- Zu einer Person gehört genau ein Impfausweis.
- Umgekehrt gehört auch zu jedem Impfausweis genau eine Person.

i>**1:n-Beziehung**<br />Einem Element der Menge A werden verschiedene Elemente der Menge B zugeordnet.

- Ein Kunde kann mehrere Aufträge erteilen.
- Umgekehrt gehört jeder Auftrag zu einem Kunden.

i>m:n-Beziehung<br />m Elemente der Menge A werden n Elementen der Menge B zugeordnet.

- In m Fabriken werden n gleiche Produkte hergestellt.
- Zu jedem Produkt k nnen mehrere Fabriken gehören.

![Kardinalität](bilder/kardinalität-beispiel.png)

---

### Übung Kardinalität

Bestimme die Kardinalitäten zu folgenden Beispielen:

![Übung](bilder/kardinalität-übung.png)

## Übung ERM

t>Zeichne zu den gegebenen Sachverhalten ein geeignetes Datenbankmodell mittels ERM. Nutze dafür das Programm *Dia* (Desktop/Informatik) oder zeichne auf Papier.

### Aufgabe 1 - Hausverwaltung

Eine Hausverwaltung möchte Ihre Wohnungen, Mieter und Mietshäuser verwalten. Ein Mietshaus hat mehrere Wohnungen und wird von einem Hausmeister betreut. Jede Wohnung ist vermietet. Es reicht, wenn du neben dem Schlüsselattribut zwei weitere Attribute angibst.

### Aufgabe 2 - Versandhaus

Kunden (charakterisiert durch Name und Kundennummer) bestellen Artikel (charakterisiert durch Artikelnummer und Bezeichnung) an einem bestimmten Datum in einer gewissen Stückzahl. Trage auch die Kardinalitäten ein. Achte auf eindeutige Schlüsselattribute und mache diese kenntlich.

### Aufgabe 3 - Schulverwaltung

Schüler besuchen Kurse, welche von Lehrern gehalten werden. Lehrer sind die Tutoren der Schüler.
Es sollen auch Noten und die Fehlstunden in den jeweiligen Kursen erfasst werden.

### Aufgabe 4 - Kindergarten

Ein Kindergarten hat jedes Jahr zwischen vier und sechs Kindergartengruppen und in jeder Gruppe ca. 20 Kinder. Von den Kindern sollen Name und Alter gespeichert werden. Außerdem braucht die Kindergartenleitung für die Statistik eine Angabe, welche Kinder mit der U-Bahn, dem Bus, dem Auto, der Tram oder sonstigen Verkehrsmitteln zum Kindergarten fährt und welches Kind im Kindergarten Frühstück und/oder Mittagessen und/oder Abendessen bekommt (die Eltern können wählen, je nachdem wie lange das Kind im Kindergarten bleibt).
Modelliere die Datenbank sinnvoll.
